from iinfer import version
from iinfer.app.commons import convert
from PIL import Image
from typing import Dict, Any
from pathlib import Path
import copy
import eel
import iinfer
import logging
import queue
import threading

class Showimg(object):
    def __init__(self, logger:logging.Logger, showroot:Path=str(Path(iinfer.__file__).parent / "showroot")):
        self.logger = logger
        eel.init(showroot)
        self.eel_sq = queue.Queue(10)

    def start(self, width:int=1080, height:int=700, web_host:str="localhost", web_port:int=8001):
        self.logger.info(f"Start eel Showimg on http://{web_host}:{web_port}")

        @eel.expose
        def copyright():
            return version.__copyright__

        @eel.expose
        def versions_iinfer():
            return version.__description__.split('\n')
    
        @eel.expose
        def versions_used():
            with open(Path(iinfer.__file__).parent / 'licenses' / 'files.txt', 'r', encoding='utf-8') as f:
                ret = []
                for i, line in enumerate(f.readlines()):
                    parts = line.strip().split('\t')
                    ret.append(parts)
            return ret

        def run_eel(eel_sq:queue.Queue):
            self.showing = True
            eel.js_console_modal_log_func('== console log start ==\n')
            def show_proc():
                while self.showing:
                    try:
                        eel.sleep(0.1)
                        params = eel_sq.get(block=True, timeout=1)
                        outputs:Dict[str, Any] = params[0]
                        output_image:Image.Image = params[1]
                        jpg = convert.img2byte(output_image)
                        jpg_url = f"data:image/jpeg;base64,{convert.bytes2b64str(jpg)}"
                        outputs = copy.deepcopy(outputs)
                        if 'output_image' in outputs:
                            del outputs['output_image']
                        if 'output_image_shape' in outputs:
                            del outputs['output_image_shape']
                        eel.js_show_func(outputs, jpg_url)
                    except queue.Empty:
                        eel.sleep(0.1)
            eel.spawn(show_proc)
            eel.start("showimg.html", size=(width, height), block=False, port=web_port, host=web_host, close_callback=self.stop)
            while self.showing:
                eel.sleep(0.1)
        self.eel_th = threading.Thread(target=run_eel, args=(self.eel_sq,))
        self.eel_th.start()

    def show(self, outputs:Dict[str, Any], output_image:Image.Image):
        self.eel_sq.put((outputs, output_image))

    def stop(self, route, websockets):
        self.showing = False
        exit(0)



from iinfer.app import web
from pathlib import Path
import bottle
import eel
import logging
import iinfer


class Gui(web.Web):
    def __init__(self, logger:logging.Logger, data:Path):
        super().__init__(logger, data)
        eel.init(str(Path(iinfer.__file__).parent / "web"))

    def start(self, width:int=1080, height:int=700, web_host:str="localhost", web_port:int=8080):
        self.logger.info(f"Start eel web on http://{web_host}:{web_port}")

        @eel.expose
        def get_local_data():
            return str(self.data)

        @eel.expose
        def get_modes():
            return self.get_modes()

        @eel.expose
        def get_cmds(mode):
            return self.get_cmds(mode)

        @eel.expose
        def get_cmd_choices(mode, cmd):
            return self.get_cmd_choices(mode, cmd)

        @eel.expose
        def list_cmd(kwd):
            return self.list_cmd(kwd)

        @eel.expose
        def save_cmd(title, opt):
            return self.save_cmd(title, opt)

        @eel.expose
        def load_cmd(title):
            return self.load_cmd(title)

        @eel.expose
        def del_cmd(title):
            self.del_cmd(title)

        @eel.expose
        def bbforce_cmd():
            self.bbforce_cmd()

        @eel.expose
        def exec_cmd(title, opt, nothread=False):
            return self.exec_cmd(title, opt, nothread)

        @eel.expose
        def raw_cmd(title, opt):
            return self.raw_cmd(title, opt)

        @eel.expose
        def list_tree(current_path):
            return self.list_tree(current_path)
        
        @eel.expose
        def load_result(current_path):
            return self.load_result(current_path)

        @eel.expose
        def load_capture(current_path):
            return self.load_capture(current_path)

        @eel.expose
        def list_pipe(kwd):
            return self.list_pipe(kwd)

        @eel.expose
        def exec_pipe(title, opt):
            return self.exec_pipe(title, opt)

        @eel.expose
        def raw_pipe(title, opt):
            return self.raw_pipe(title, opt)

        @eel.expose
        def save_pipe(title, opt):
            return self.save_pipe(title, opt)

        @eel.expose
        def del_pipe(title):
            self.del_pipe(title)

        @eel.expose
        def load_pipe(title):
            return self.load_pipe(title)

        @eel.expose
        def copyright():
            return self.copyright()

        @eel.expose
        def versions_iinfer():
            return self.versions_iinfer()

        @eel.expose
        def versions_used():
            return self.versions_used()
        
        @bottle.route('/filer/upload', method='POST')
        def filer_upload():
            return self.filer_upload(bottle.request)

        eel.js_console_modal_log_func('== console log start ==\n')
        eel.start("gui.html", size=(width, height), block=True, port=web_port, host=web_host, close_callback=self.stop)

    def callback_console_modal_log_func(self, output:dict):
        eel.js_console_modal_log_func(output)
    
    def callback_return_cmd_exec_func(self, title, output:dict):
        eel.js_return_cmd_exec_func(title, output)

    def callback_return_pipe_exec_func(self, title, output):
        eel.js_return_pipe_exec_func(title, output)

    def callback_return_stream_log_func(self, output):
        eel.js_return_stream_log_func(output)

    def stop(self, route, websockets):
        self.bbforce_cmd()
        self.logger.info(f"Stop eel web. {route}")
        exit(0)




from iinfer.app import postprocess
from iinfer.app.commons import convert
from iinfer.app.injections import after_http_injection
from PIL import Image
from typing import Dict, Any
from urllib3.exceptions import InsecureRequestWarning
import logging
import requests
import urllib3
urllib3.disable_warnings(InsecureRequestWarning)

class Httpreq(postprocess.Postprocess):
    def __init__(self, logger:logging.Logger, fileup_name:str='file', json_without_img:bool=False):
        """
        HTTP Requestを行う後処理クラスです。
        
        Args:
            logger (logging.Logger): ロガー
            fileup_name (str, optional): ファイルアップロード名。デフォルトは'file'。
            json_without_img (bool, optional): JSONに画像を含めない場合はTrue。デフォルトはFalse。
        """
        super().__init__(logger, json_without_img)
        self.fileup_name = fileup_name
        self.config = dict(fileup_name=fileup_name, json_without_img=json_without_img)
        self.injection = after_http_injection.AfterHttpInjection(self.config, self.logger)

    def create_session(self, json_connectstr:str, img_connectstr:str, text_connectstr:str):
        """
        後処理のセッションを作成する関数です。
        ここで後処理準備を完了するようにしてください。
        戻り値の後処理セッションの型は問いません。

        Args:
            json_connectstr (str): 推論結果後処理のセッション確立に必要な接続文字列
            img_connectstr (str): 可視化画像後処理のセッション確立に必要な接続文字列
            text_connectstr (str): テキストデータ処理のセッション確立に必要な接続文字列

        Returns:
            推論結果後処理のセッション
            可視化画像後処理のセッション
            テキストデータ処理のセッション
        """
        self.injection._set_config('outputs_url', json_connectstr)
        self.injection._set_config('output_image_url', img_connectstr)
        self.req_session = requests.Session()
        return json_connectstr, img_connectstr, text_connectstr

    def post_text(self, text_session, res_str:str):
        """
        res_strに対して後処理を行う関数です。

        Args:
            text_session (任意): テキストセッション
            res_str (text): 入力テキスト

        Returns:
            str: 後処理結果
        """
        if text_session is None:
            return res_str
        result = self.injection.post_text(text_session, self.req_session, res_str)
        return result

    def post_json(self, json_session, outputs:Dict[str, Any], output_image:Image.Image):
        """
        outputsに対して後処理を行う関数です。

        Args:
            json_session (任意): JSONセッション
            outputs (Dict[str, Any]): 推論結果
            output_image (Image.Image): 入力画像（RGB配列であること）

        Returns:
            Dict[str, Any]: 後処理結果
        """
        if json_session is None:
            return outputs
        result = self.injection.post_json(json_session, self.req_session, outputs)
        return result

    def post_img(self, img_session, result:Dict[str, Any], output_image:Image.Image):
        """
        output_imageに対して後処理を行う関数です。
        引数のimageはRGBですので、戻り値の出力画像もRGBにしてください。

        Args:
            img_session (任意): 画像セッション
            result (Dict[str, Any]): 後処理結果
            output_image (Image.Image): 入力画像（RGB配列であること）

        Returns:
            Image: 後処理結果
        """
        if img_session is None:
            return output_image
        result = self.injection.post_img(img_session, self.req_session, result, output_image)
        return result



from iinfer.app import postprocess
from iinfer.app.injections import after_csv_injection
from PIL import Image
from typing import Dict, List, Any
import logging

class Csv(postprocess.Postprocess):
    def __init__(self, logger:logging.Logger, out_headers:List[str]=None, noheader:bool=False, json_without_img:bool=False):
        """
        JSONをCSVに変換する後処理クラスです。
        
        Args:
            logger (logging.Logger): ロガー
            out_headers (List[str], optional): 出力させるヘッダー. Defaults to None.
            noheader (bool, optional): ヘッダーを出力しない. Defaults to False.
            json_without_img (bool, optional): JSONに画像を含めない場合はTrue. Defaults to False.
        """
        super().__init__(logger, json_without_img)
        self.config = dict(out_headers=out_headers, noheader=noheader)
        self.injection = after_csv_injection.AfterCSVInjection(self.config, self.logger)

    def create_session(self, json_connectstr:str, img_connectstr:str, text_connectstr:str):
        """
        後処理のセッションを作成する関数です。
        ここで後処理準備を完了するようにしてください。
        戻り値の後処理セッションの型は問いません。

        Args:
            json_connectstr (str): 推論結果後処理のセッション確立に必要な接続文字列
            img_connectstr (str): 可視化画像後処理のセッション確立に必要な接続文字列
            text_connectstr (str): テキストデータ処理のセッション確立に必要な接続文字列

        Returns:
            推論結果後処理のセッション
            可視化画像後処理のセッション
            テキストデータ処理のセッション
        """
        return 'json_connectstr', None, None

    def post_json(self, json_session, outputs:Dict[str, Any], output_image:Image.Image):
        """
        outputsに対して後処理を行う関数です。

        Args:
            json_session (任意): JSONセッション
            outputs (Dict[str, Any]): 推論結果
            output_image (Image.Image): 入力画像（RGB配列であること）

        Returns:
            Dict[str, Any]: 後処理結果
        """
        result = self.injection.write_csv(outputs, self.config['out_headers'], self.config['noheader'])
        self.config['noheader'] = True
        return result



from iinfer.app import postprocess
from iinfer.app.commons import convert
from pathlib import Path
from PIL import Image
from typing import Dict, Any
import cv2
import logging
import numpy as np


class DetClip(postprocess.Postprocess):
    def __init__(self, logger:logging.Logger, image_type:str='capture', clip_margin:int=0, json_without_img:bool=False):
        """
        Object Detectionの推論結果となったbbox部分を個別の画像として切り出す後処理クラスです。
        
        Args:
            logger (logging.Logger): ロガー
            image_type (str): 切り出した画像のファイル形式
            clip_margin (int): bboxの周囲に余白を設けるピクセル数
            output_dir (Path): 切り出した画像を保存するディレクトリ
            json_without_img (bool, optional): JSONに画像を含めない場合はTrue。デフォルトはFalse。
        """
        super().__init__(logger, json_without_img)
        self.image_type = image_type
        self.clip_margin = clip_margin

    def create_session(self, json_connectstr:str, img_connectstr:str, text_connectstr:str):
        """
        後処理のセッションを作成する関数です。
        ここで後処理準備を完了するようにしてください。
        戻り値の後処理セッションの型は問いません。

        Args:
            json_connectstr (str): 推論結果後処理のセッション確立に必要な接続文字列
            img_connectstr (str): 可視化画像後処理のセッション確立に必要な接続文字列
            text_connectstr (str): テキストデータ処理のセッション確立に必要な接続文字列

        Returns:
            推論結果後処理のセッション
            可視化画像後処理のセッション
            テキストデータ処理のセッション
        """
        return 'json_connectstr', None, None

    def post_json(self, json_session, outputs:Dict[str, Any], output_image:Image.Image):
        """
        outputsに対して後処理を行う関数です。

        Args:
            json_session (任意): JSONセッション
            outputs (Dict[str, Any]): 推論結果
            output_image (Image.Image): 入力画像（RGB配列であること）

        Returns:
            Dict[str, Any]: 後処理結果
        """
        if 'success' not in outputs or type(outputs['success']) != dict:
            raise Exception('Invalid outputs. outputs[\'success\'] must be dict.')
        data = outputs['success']
        if 'output_boxes' not in data:
            raise Exception('Invalid outputs. outputs[\'success\'][\'output_boxes\'] must be set.')
        output_boxes = data['output_boxes']

        if output_image is None or "output_image_name" not in outputs:
            raise Exception('Invalid outputs. outputs[\'success\'][\'output_image\'] and outputs[\'success\'][\'output_image_shape\'] and outputs[\'success\'][\'output_image_name\'] must be set.')

        output_image_name = Path(outputs["output_image_name"]).stem

        # 切り出し
        result = ''
        for i, box in enumerate(output_boxes):
            y1, x1, y2, x2 = box
            x1 = max(0, np.floor(x1 + 0.5).astype(int) - self.clip_margin)
            y1 = max(0, np.floor(y1 + 0.5).astype(int) - self.clip_margin)
            x2 = min(output_image.width, np.floor(x2 + 0.5).astype(int) + self.clip_margin)
            y2 = min(output_image.height, np.floor(y2 + 0.5).astype(int) + self.clip_margin)

            cropped_image = output_image.crop((x1, y1, x2, y2))
            img_npy = convert.img2npy(cropped_image)

            img_b64 = None
            image_name = f"{output_image_name}.{i}.{self.image_type}"
            if self.image_type == 'capture' or self.image_type is None:
                img_b64 = convert.npy2b64str(img_npy)
            else:
                img_byte = convert.img2byte(cropped_image, format=self.image_type)
                img_b64 = convert.bytes2b64str(img_byte)
            result += f'{self.image_type},'+img_b64+f',{img_npy.shape[0]},{img_npy.shape[1]},{img_npy.shape[2] if len(img_npy.shape) > 2 else -1},{image_name}\n'

        return result
    